package api

import (
	"assetcenter/db/model"
	"assetcenter/db/repo"
	"assetcenter/utils"
	"fmt"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

func CreateDatabaseAssetConfRouter(router *GinRouter) {

	//创建资产定义
	router.POST("/assetconf/create/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}
		body := &comm.DbAsset{}
		c.ShouldBindJSON(body)
		body.Id = primitive.NewObjectID().Hex()

		if len(body.Label) < 1 || body.Type == 0 {
			return nil, NewError("数据不合法!")
		}
		body.CreateTime = time.Now()

		//数据库名字添加一个随机数
		var createCollRandomName = func(assetType int) string {
			typePrefixMap := map[int]string{}
			typePrefixMap[model.AssetTypeMesh] = "mesh"
			typePrefixMap[model.AssetTypeImage] = "image"
			typePrefixMap[model.AssetTypeMaterial] = "material"
			typePrefixMap[model.AssetTypeMaterialGroup] = "matgroup"
			typePrefixMap[model.AssetTypeEnv3d] = "hdr"
			typePrefixMap[model.AssetTypePackage] = "scene"
			prefix := typePrefixMap[assetType]
			if len(prefix) < 1 {
				return ""
			}
			return fmt.Sprintf("%s-%s-%s", prefix, time.Now().Format("20060102"), utils.RandName(6))
		}
		coll := createCollRandomName(body.Type)
		if len(coll) < 1 {
			return nil, fmt.Errorf("不支持的资产类型")
		}
		body.Collection = coll
		database := &comm.Database{}
		ok, _ := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionDatabase, Query: repo.Map{"assets.collection": body.Collection, "_id": id}, Project: []string{"_id"}}, database)
		if ok {
			return nil, NewError("资产库编号不能重复")
		}

		return repo.RepoDocArrayAppend(
			apictx.CreateRepoCtx(),
			repo.CollectionDatabase,
			id,
			"assets",
			body,
		)
	})

	//创建资产定义
	router.GET("/assetconf/list/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}

		database := &comm.Database{}

		_, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionDatabase, Query: repo.Map{"_id": id}, Project: []string{"assets"}}, database)

		return database.Assets, err
	})

	//删除资产定义
	router.POST("/assetconf/delete/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}
		body := &comm.DbAsset{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 {
			return nil, NewError("资产Id不能为空!")
		}

		return repo.RepoDocArrayOneRemove(
			apictx.CreateRepoCtx(),
			&repo.ArrayOneRemoveOption{
				CollectName: repo.CollectionDatabase,
				Id:          id,
				ArrayQuery:  repo.Map{"assets": bson.M{"id": body.Id}},
			},
		)
	})

	//更新资产标签
	router.POST("/assetconf/update/label/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}
		body := &comm.DbAsset{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 || len(body.Label) < 1 {
			return nil, NewError("参数非法")
		}
		optSet := repo.Map{}
		optSet["assets.$.label"] = body.Label
		option := &repo.ArrayOneUpdateOption{
			CollectName: repo.CollectionDatabase,
			Id:          id,
			Query:       repo.Map{"assets.id": body.Id},
			Set:         optSet,
		}
		return repo.RepoDocArrayOneUpdate(
			apictx.CreateRepoCtx(),
			option,
		)
	})

	router.POST("/assetconf/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}

		body := &comm.DbAsset{}
		c.ShouldBindJSON(body)

		if len(body.Id) < 1 || len(body.Label) < 1 {
			return nil, NewError("参数非法")
		}
		optSet := repo.Map{}

		if len(body.Label) > 0 {
			optSet["assets.$.label"] = body.Label
		}
		if body.CategoryIds != nil {
			optSet["assets.$.categoryIds"] = body.CategoryIds
		}
		option := &repo.ArrayOneUpdateOption{
			CollectName: repo.CollectionDatabase,
			Id:          id,
			Query:       repo.Map{"assets.id": body.Id},
			Set:         optSet,
		}

		return repo.RepoDocArrayOneUpdate(
			apictx.CreateRepoCtx(),
			option,
		)
	})

	//更新资产分类
	router.POST("/assetconf/update/categoryIds/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("数据库Id不能为空")
		}
		body := &comm.DbAsset{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 {
			return nil, NewError("参数非法")
		}
		optSet := repo.Map{}
		optSet["assets.$.categoryIds"] = body.CategoryIds
		option := &repo.ArrayOneUpdateOption{
			CollectName: repo.CollectionDatabase,
			Id:          id,
			Query:       repo.Map{"assets.id": body.Id},
			Set:         optSet,
		}
		return repo.RepoDocArrayOneUpdate(
			apictx.CreateRepoCtx(),
			option,
		)
	})
}
